<?php

namespace Drupal\monarch_utilities\Utility;

use DateTime;
use DateTimeZone;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\FieldableEntityInterface;


/**
 * Get the value of an entity field.
 *
 * @package Drupal\webportal_service\Utility
 */
class EntityFieldValue {

  /**
   * @var \Drupal\Core\Entity\FieldableEntityInterface
   */
  protected $entity;

  /**
   * @var string
   */
  protected $field_name;

  /**
   * @var integer
   */
  protected $delta;

  /**
   * @var string
   */
  protected $column;

  /**
   * EntityFieldValue constructor.
   *
   * @param \FieldableEntityInterface $entity
   *   The Fieldable Entity object.
   * @param string $field_name
   *   The machine name of the field to load.
   * @param int $delta
   *   The delta position for multi-value fields. Default is 0.
   * @param string $column
   *   The name of the column to retrieve. Default is 'value'.
   */
  public function __construct(FieldableEntityInterface $entity, $field_name, int  $delta = 0, string $column = 'value') {
    $this->entity = $entity;
    $this->field_name = $field_name;
    $this->delta = $delta;
    $this->column = $column;
  }

  /**
   * Get the value of the entity field as a string.
   *
   * @return string The field value.
   */
  public function getStringValue() {
    if ($this->entity->hasField($this->field_name)) {
      $field = $this->entity->get($this->field_name)->getValue();
      if (isset($field[$this->delta][$this->column])) {
        return $field[$this->delta][$this->column];
      }
    }
    return '';
  }

  /**
   * Get the value of the entity field as a boolean.
   *
   * @return boolean The field value.
   */
  public function getBooleanValue() {
    if ($this->entity->hasField($this->field_name)) {
      $field = $this->entity->get($this->field_name)->getValue();
      if (isset($field[$this->delta][$this->column])) {
        return !empty($field[$this->delta][$this->column]);
      }
    }
    return FALSE;
  }

  /**
   * Get the string value of a entity date field.
   *
   * @param string $format
   *   The date format to return.
   * @param DateTimeZone|Null 
   *   The timezone of the source date field or null for UTC (default)
   *
   * @return string The field value.
   */
  public function getDateStringValue($format = 'm/d/Y', DateTimeZone $timezone = NULL) {
    $date = $this->getDateTimeObject($timezone);
    if (!empty($date)) {
      return $date->format($format);
    }
    return '';
  }

  /**
   * Get the datetime object value of a entity date field.
   *
   * @param DateTimeZone|Null 
   *   The timezone of the source date field or null for UTC (default)
   *
   * @return DrupalDateTime The field datetime object.
   */
  public function getDateTimeObject(DateTimeZone $timezone = NULL) {
    if ($this->entity->hasField($this->field_name)) {
      $field = $this->entity->get($this->field_name);
      $definition = $field->getFieldDefinition();
      if ($definition->getType() == 'datetime' || $definition->getType() == 'daterange') {
        $field_value = $field->getValue();
        if (isset($field_value[$this->delta][$this->column])) {
          if (empty($timezone)) {
            $timezone = new DateTimeZone('UTC');
          }

          // Determine type
          $settings = $definition->getSettings();
          $formats = [
            'datetime' => 'Y-m-d\TH:i:s',
            'date' => 'Y-m-d',
          ];

          return DrupalDateTime::createFromFormat($formats[$settings['datetime_type']], $field_value[$this->delta][$this->column], $timezone);
        }
      }
    }
    return '';
  }

  /**
   * Get an Entity reference target ID.
   */
  public function getTargetID() {
    if ($this->entity->hasField($this->field_name)) {
      $field = $this->entity->get($this->field_name)->getValue();
      if (isset($field[$this->delta]['target_id'])) {
        return $field[$this->delta]['target_id'];
      }
    }
    return null;
  }

}
